@@ -3,9 +3,10 @@
     Author:        Benoit Desouter <Benoit.Desouter@UGent.be>
                    Jan Wielemaker (SWI-Prolog port)
                    Fabrizio Riguzzi (mode directed tabling)
-    Copyright (c) 2016-2020, Benoit Desouter,
+    Copyright (c) 2016-2021, Benoit Desouter,
                              Jan Wielemaker,
                              Fabrizio Riguzzi
+                             SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -377,7 +378,7 @@ start_tabling_2(Closure, Wrapper, Worker, Trie, Status, Skeleton) :-
     ;   Status == invalid
     ->  reeval(Trie, Wrapper, Skeleton)
     ;   % = run_follower, but never fresh and Status is a worklist
-        shift(call_info(Skeleton, Status))
+        shift_for_copy(call_info(Skeleton, Status))
     ).
 
 create_table(Trie, Fresh, Skeleton, Wrapper, Worker) :-
@@ -429,7 +430,7 @@ start_subsumptive_tabling(Closure, Wrapper, Worker) :-
         ;   Status == invalid
         ->  reeval(Trie, Wrapper, Skeleton),
             trie_gen_compiled(Trie, Skeleton)
-        ;   shift(call_info(Skeleton, Status))
+        ;   shift_for_copy(call_info(Skeleton, Status))
         )
     ;   more_general_table(Wrapper, ATrie),
         '$tbl_table_status'(ATrie, complete, Wrapper, Skeleton)
@@ -441,7 +442,7 @@ start_subsumptive_tabling(Closure, Wrapper, Worker) :-
             Wrapper = GenWrapper,
             '$tbl_answer_update_dl'(ATrie, GenSkeleton)
         ;   wrapper_skeleton(GenWrapper, GenSkeleton, Wrapper, Skeleton),
-            shift(call_info(GenSkeleton, Skeleton, Status)),
+            shift_for_copy(call_info(GenSkeleton, Skeleton, Status)),
             unify_subsumptive(Skeleton, GenSkeleton)
         )
     ;   start_tabling(Closure, Wrapper, Worker)
@@ -490,7 +491,7 @@ start_abstract_tabling(Closure, Wrapper, Worker) :-
         reeval(ATrie, GenWrapper, GenSkeleton),
         Wrapper = GenWrapper,
         '$tbl_answer_update_dl'(ATrie, Skeleton)
-    ;   shift(call_info(GenSkeleton, Skeleton, Status)),
+    ;   shift_for_copy(call_info(GenSkeleton, Skeleton, Status)),
         unify_subsumptive(Skeleton, GenSkeleton)
     ).
 
@@ -576,7 +577,7 @@ run_leader(Skeleton, Worker, fresh(SCC, Worklist), Status, Clause) :-
     (   Status == merged
     ->  tdebug(merge, 'Turning leader ~p into follower', [Goal]),
         '$tbl_wkl_make_follower'(Worklist),
-        shift(call_info(Skeleton, Worklist))
+        shift_for_copy(call_info(Skeleton, Worklist))
     ;   true                                    % completed
     ).
 
@@ -681,7 +682,7 @@ start_moded_tabling_2(_Closure, Wrapper, Worker, ModeArgs,
     ->  reeval(Trie, Wrapper, Skeleton),
         moded_gen_answer(Trie, Skeleton, ModeArgs)
     ;   % = run_follower, but never fresh and Status is a worklist
-        shift(call_info(Skeleton/ModeArgs, Status))
+        shift_for_copy(call_info(Skeleton/ModeArgs, Status))
     ).
 
 :- public
@@ -714,7 +715,7 @@ moded_run_leader(Wrapper, SkeletonMA, Worker, fresh(SCC, Worklist), Status) :-
     (   Status == merged
     ->  tdebug(merge, 'Turning leader ~p into follower', [Wrapper]),
         '$tbl_wkl_make_follower'(Worklist),
-        shift(call_info(SkeletonMA, Worklist))
+        shift_for_copy(call_info(SkeletonMA, Worklist))
     ;   true                                    % completed
     ).
 
@@ -890,7 +891,7 @@ floundering(Goal) :-
 negation_suspend(Wrapper, Skeleton, Worklist) :-
     tdebug(tnot, 'negation_suspend ~p (wl=~p)', [Wrapper, Worklist]),
     '$tbl_wkl_negative'(Worklist),
-    shift(call_info(Skeleton, tnot(Worklist))),
+    shift_for_copy(call_info(Skeleton, tnot(Worklist))),
     tdebug(tnot, 'negation resume ~p (wl=~p)', [Wrapper, Worklist]),
     '$tbl_wkl_is_false'(Worklist).
 
@@ -1871,19 +1872,19 @@ do_reeval(ATrie, Goal, Return) :-
 %!  clean_paths(+PathsIn, -Paths)
 %
 %   Clean the reevaluation paths. Get rid of   the head term for ranking
-%   and remove duplicate paths.
+%   and remove duplicate paths. Note that  a   Path  is a list of tries,
+%   ground terms.
 
 clean_paths([], []).
 clean_paths([[_|Path]|T0], [Path|T]) :-
     clean_paths(T0, Path, T).
 
 clean_paths([], _, []).
-clean_paths([[_|Path]|T0], CPath, T) :-
-    Path =@= CPath,
+clean_paths([[_|CPath]|T0], CPath, T) :-
     !,
     clean_paths(T0, CPath, T).
-clean_paths([[_|Path]|T0], CPath, [Path|T]) :-
-    clean_paths(T0, CPath, T).
+clean_paths([[_|Path]|T0], _, [Path|T]) :-
+    clean_paths(T0, Path, T).
 
 %!  reeval_paths(+Paths, +Atrie)
 %
@@ -1898,24 +1899,26 @@ reeval_paths(BottomUp, ATrie) :-
     is_invalid(ATrie),
     !,
     reeval_heads(BottomUp, ATrie, BottomUp1),
-    reeval_paths(BottomUp1, ATrie).
+    tdebug(assertion(BottomUp \== BottomUp1)),
+    '$list_to_set'(BottomUp1, BottomUp2),
+    reeval_paths(BottomUp2, ATrie).
 reeval_paths(_, _).
 
-reeval_heads(_, ATrie, _) :-
+reeval_heads(_, ATrie, []) :-                % target is valid again
     \+ is_invalid(ATrie),
     !.
 reeval_heads([], _, []).
-reeval_heads([[H]|B], ATrie, BT) :-
-    !,
+reeval_heads([[H]|B], ATrie, BT) :-          % Last one of a falsepath
     reeval_node(H),
-    reeval_heads(B, ATrie, BT).
-reeval_heads([[]|B], ATrie, BT) :-
     !,
     reeval_heads(B, ATrie, BT).
 reeval_heads([[H|T]|B], ATrie, [T|BT]) :-
-    !,
     reeval_node(H),
+    !,
     reeval_heads(B, ATrie, BT).
+reeval_heads([FP|B], ATrie, [FP|BT]) :-
+    reeval_heads(B, ATrie, BT).
+
 
 %!  false_path(+Atrie, -Path) is nondet.
 %
@@ -1956,7 +1959,7 @@ is_invalid(ATrie) :-
     '$idg_falsecount'(ATrie, FalseCount),
     FalseCount > 0.
 
-%!  reeval_node(+ATrie)
+%!  reeval_node(+ATrie) is semidet.
 %
 %   Re-evaluate the invalid answer trie ATrie.  Initially this created a
 %   nested tabling environment, but this is dropped:
@@ -1966,6 +1969,9 @@ is_invalid(ATrie) :-
 %       outer SCC.  This doesn't work well with a sub-environment.
 %     - We do not need one.  If this environment is not merged into the
 %       outer one it will complete before we continue.
+%
+%   Fails if the node is not ready for   evaluation. This is the case if
+%   it is valid or it is a lazy table that has invalid dependencies.
 
 reeval_node(ATrie) :-
     '$tbl_reeval_prepare'(ATrie, M:Variant),
@@ -1988,11 +1994,13 @@ reeval_node(ATrie) :-
         '$tbl_propagate_start'(Old),
         reeval_monotonic_node(ATrie, Size),
         '$tbl_propagate_end'(Old)).
-reeval_node(_).
+reeval_node(ATrie) :-
+    \+ is_invalid(ATrie).
 
 reeval_monotonic_node(ATrie, Size) :-
     tdebug(reeval, 'Re-evaluating lazy monotonic ~p', [ATrie]),
     (   '$idg_mono_affects_lazy'(ATrie, _0SrcTrie, Dep, DepRef, Answers),
+        length(Answers, Count),
         (   Dep = dependency(Head, Cont, Skel)
         ->  (   '$member'(ClauseRef, Answers),
                 '$clause'(Head, _Body, ClauseRef, _Bindings),
@@ -2000,7 +2008,7 @@ reeval_monotonic_node(ATrie, Size) :-
                        [Head, _0SrcTrie, ATrie]),
                 pdelim(Cont, Skel, ATrie),
                 fail
-            ;   '$idg_mono_empty_queue'(DepRef)
+            ;   '$idg_mono_empty_queue'(DepRef, Count)
             )
         ;   Dep = dependency(SrcSkel, true, Cont, Skel)
         ->  (   '$member'(Node, Answers),
@@ -2009,15 +2017,27 @@ reeval_monotonic_node(ATrie, Size) :-
                        [Skel, _0SrcTrie, ATrie]),
                 pdelim(Cont, Skel, ATrie),
                 fail
-            ;   '$idg_mono_empty_queue'(DepRef)
+            ;   '$idg_mono_empty_queue'(DepRef, Count)
             )
         ;   tdebug(monotonic, 'Skipped queued ~p, answers ~p',
                    [Dep, Answers])
         ),
         fail
-    ;   '$mono_reeval_done'(ATrie, Size)
+    ;   '$mono_reeval_done'(ATrie, Size, Deps),
+        (   Deps == []                % all done
+        ->  true
+        ;   Deps == false             % re-evaluation queued new answers
+        ->  reeval_node(ATrie)
+        ;   reeval_nodes(Deps),       % not all dependents are valid
+            reeval_node(ATrie)
+        )
     ).
 
+reeval_nodes([]).
+reeval_nodes([H|T]) :-
+    reeval_node(H),
+    reeval_nodes(T).
+
 
 		 /*******************************
 		 *      EXPAND DIRECTIVES	*
