@@ -45,6 +45,7 @@
             pack_rebuild/0,             % All packages
             pack_remove/1,              % +Name
             pack_property/2,            % ?Name, ?Property
+            pack_attach/2,              % +Dir, +Options
 
             pack_url_file/2             % +URL, -File
           ]).
@@ -2298,6 +2299,30 @@ string([]) --> [].
 string([H|T]) --> [H], string(T).
 
 
+%!  pack_attach(+Dir, +Options) is det.
+%
+%   Attach a single package in Dir.  The Dir is expected to contain
+%   the file `pack.pl` and a `prolog` directory.  Options processed:
+%
+%     - duplicate(+Action)
+%     What to do if the same package is already installed in a different
+%     directory.  Action is one of
+%       - warning
+%       Warn and ignore the package
+%       - keep
+%       Silently ignore the package
+%       - replace
+%       Unregister the existing and insert the new package
+%     - search(+Where)
+%     Determines the order of searching package library directories.
+%     Default is `last`, alternative is `first`.
+%
+%   @see attach_packs/2 to attach multiple packs from a directory.
+
+pack_attach(Dir, Options) :-
+    '$pack_attach'(Dir, Options).
+
+
                  /*******************************
                  *        USER INTERACTION      *
                  *******************************/
