@@ -165,7 +165,7 @@ oututf8(format_state *state, const char *s, size_t len)
   while(s<e)
   { int chr;
 
-    s = utf8_get_char(s, &chr);
+    PL_utf8_code_point(&s, e, &chr);
     if ( !outchr(state, chr) )
       return FALSE;
   }
@@ -601,7 +601,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		  } else			/* r,R */
 		  { if ( arg == DEFAULT )
 		      FMT_ERROR("r,R requires radix specifier");
-		    if ( arg < 1 || arg > 36 )
+		    if ( arg < 2 || arg > 36 )
 		    { term_t r = PL_new_term_ref();
 
 		      PL_put_integer(r, arg);
@@ -915,7 +915,7 @@ emit_rubber(format_state *state)
     }
 
     if ( s < e )
-    { s = utf8_get_char(s, &chr);
+    { PL_utf8_code_point(&s, e, &chr);
       if ( Sputcode(chr, state->out) < 0 )
 	return FALSE;
     } else
