@@ -122,9 +122,9 @@ typedef struct
 { tmp_buffer	pattern;
 } compiled_pattern;
 
-static char *compile_pattern(compiled_pattern *pattern, char *s,
-			     int curl, int flags);
-static int match_pattern(matchcode *pattern, char *s, int flags);
+static const char *compile_pattern(compiled_pattern *pattern, const char *s,
+				   int curl, int flags);
+static int match_pattern(matchcode *pattern, const char *s, int flags);
 
 #define Output(c)  addBuffer(&Out->pattern, c, matchcode)
 #define Here()	   entriesBuffer(&Out->pattern, matchcode)
@@ -153,23 +153,23 @@ compilePattern(char *p, compiled_pattern *cbuf, int flags)
 }
 
 static int
-utf8_peek(char *p, int n)
+utf8_peek(const char *p, int n)
 { int c;
 
   while(n-- > 0)
     p = utf8_skip_char(p);
 
-  utf8_get_char(p, &c);
+  PL_utf8_code_point(&p, NULL, &c);
 
   return c;
 }
 
-static char *
-compile_pattern(compiled_pattern *Out, char *p, int curl, int mflags)
+static const char *
+compile_pattern(compiled_pattern *Out, const char *p, int curl, int mflags)
 { int c;
 
   for(;;)
-  { p = utf8_get_char(p, &c);
+  { PL_utf8_code_point(&p, NULL, &c);
 
     switch(c)
     { case EOS:
@@ -178,7 +178,7 @@ compile_pattern(compiled_pattern *Out, char *p, int curl, int mflags)
       case '\\':
       { int c2;
 
-	p = utf8_get_char(p, &c2);
+	PL_utf8_code_point(&p, NULL, &c2);
 	Output(c2 == EOS ? c : c2);
 	if ( c2 == EOS )
 	  break;
@@ -198,10 +198,7 @@ compile_pattern(compiled_pattern *Out, char *p, int curl, int mflags)
 	Output(0);
 
 	for(;;)
-	{ int cn;
-
-	  p = utf8_get_char(p, &c);
-	  utf8_get_char(p, &cn);
+	{ PL_utf8_code_point(&p, NULL, &c);
 
 	  switch( c )
 	  { case EOS:
@@ -209,7 +206,7 @@ compile_pattern(compiled_pattern *Out, char *p, int curl, int mflags)
 	    case '\\':
 	    { int c2;
 
-	      p = utf8_get_char(p, &c2);
+	      PL_utf8_code_point(&p, NULL, &c2);
 	      if ( c2 == EOS )
 		return PL_syntax_error("Unmatched '['", NULL),NULL;
 	      Output(c2);
@@ -303,7 +300,7 @@ matchPattern(char *s, compiled_pattern *cbuf, int flags)
 }
 
 static bool
-match_pattern(matchcode *p, char *s, int flags)
+match_pattern(matchcode *p, const char *s, int flags)
 { matchcode c;
 
   for(;;)
@@ -320,7 +317,7 @@ match_pattern(matchcode *p, char *s, int flags)
 	int sz = *p++;
 	matchcode *anyend = &p[sz];
 
-	s = utf8_get_char(s, &c2);
+	PL_utf8_code_point(&s, NULL, &c2);
 	if ( (flags&M_IGNCASE) )
 	  c2 = makeLowerW(c2);
 
@@ -346,7 +343,7 @@ match_pattern(matchcode *p, char *s, int flags)
 	do
 	{ if ( match_pattern(p, s, flags) )
 	    return TRUE;
-	  s = utf8_get_char(s, &c2);
+	  PL_utf8_code_point(&s, NULL, &c2);
 	} while(c2);
 
 	return FALSE;
@@ -362,7 +359,7 @@ match_pattern(matchcode *p, char *s, int flags)
       default:						/* character */
       { int c2;
 
-	s = utf8_get_char(s, &c2);
+	PL_utf8_code_point(&s, NULL, &c2);
 	if ( (flags&M_IGNCASE) )
 	  c2 = makeLowerW(c2);
 
@@ -536,7 +533,9 @@ un_escape(char *to, const char *from, const char *end)
 
 static int
 utf8_exists_file(const char *name ARG_LD)
-{ PL_chars_t txt;
+{
+#ifndef __WINDOWS__
+  PL_chars_t txt;
   int rc;
 
   PL_STRINGS_MARK();
@@ -546,17 +545,22 @@ utf8_exists_file(const char *name ARG_LD)
   txt.storage   = PL_CHARS_HEAP;
   txt.canonical = FALSE;
   rc = ( PL_canonicalise_text(&txt) &&
-	 PL_mb_text(&txt, REP_MB) &&
+	 PL_mb_text(&txt, REP_FN) &&
 	 AccessFile(txt.text.t, ACCESS_EXIST) );
   PL_free_text(&txt);
   PL_STRINGS_RELEASE();
 
   return rc;
+#else
+  return AccessFile(name, ACCESS_EXIST);
+#endif
 }
 
 static DIR*
 utf8_opendir(const char *name ARG_LD)
-{ PL_chars_t txt;
+{
+#ifndef __WINDOWS__
+  PL_chars_t txt;
   DIR *rc;
 
   PL_STRINGS_MARK();
@@ -566,7 +570,7 @@ utf8_opendir(const char *name ARG_LD)
   txt.storage   = PL_CHARS_HEAP;
   txt.canonical = FALSE;
   if ( PL_canonicalise_text(&txt) &&
-       PL_mb_text(&txt, REP_MB) )
+       PL_mb_text(&txt, REP_FN) )
     rc = opendir(txt.text.t);
   else
     rc = NULL;
@@ -574,6 +578,9 @@ utf8_opendir(const char *name ARG_LD)
   PL_STRINGS_RELEASE();
 
   return rc;
+#else
+  return opendir(name);
+#endif
 }
 
 static int
