@@ -1037,9 +1037,9 @@ pl_garbage_collect_atoms(void)
   blockSignals(&set);
   t = CpuTime(CPU_USER);
   unmarkAtoms();
-  markAtomsOnStacks(LD);
+  markAtomsOnStacks(LD, NULL);
 #ifdef O_PLMT
-  forThreadLocalDataUnsuspended(markAtomsOnStacks, 0);
+  forThreadLocalDataUnsuspended(markAtomsOnStacks, NULL);
   markAtomsMessageQueues();
 #endif
   oldcollected = GD->atoms.collected;
@@ -1716,7 +1716,8 @@ current_blob(term_t a, term_t type, frg_code call, intptr_t state ARG_LD)
 	   (!type_name || type_name == btype->atom_name) &&
 	   atom->atom != ATOM_garbage_collected &&
 	   bump_atom_references(atom, refs) )
-      { DEBUG(CHK_SECURE,	/* avoid trap through linkVal__LD() check */
+      { DEBUG(CHK_ATOM_GARBAGE_COLLECTED,
+	      /* avoid trap through linkVal__LD() check */
 	      if ( atom->atom == ATOM_garbage_collected )
 	      { PL_unregister_atom(atom->atom);
 	        continue;
