@@ -3,9 +3,10 @@
     Author:        Keri Harris
     E-mail:        keri.harris@securitease.com
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2020, University of Amsterdam
+    Copyright (c)  2011-2021, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -103,10 +104,12 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_REC_ATTVAR),
   DEBUG_TOPIC(MSG_TTY),
   DEBUG_TOPIC(MSG_OS_DIR),
+  DEBUG_TOPIC(MSG_COMP_ARG_UNIFY),
 						/* Parser */
   DEBUG_TOPIC(MSG_READ_TOKEN),
 
   DEBUG_TOPIC(MSG_CONTINUE),
+  DEBUG_TOPIC(MSG_SHIFT_CHOICE),
 
   DEBUG_TOPIC(MSG_CLEANUP_THREAD),
   DEBUG_TOPIC(MSG_INITIALISE),
