@@ -874,7 +874,7 @@ with one operation, it turns out to be faster as well.
 #define HIDE_CHILDS		(0x00800000) /* Hide children from tracer */
 #define SPY_ME			(0x01000000) /* Spy point placed */
 #define TRACE_ME		(0x02000000) /* Can be debugged */
-#define P_INCREMENTAL		(0x04000000) /* Incremental tabling */
+#define P_DET			(0x04000000) /* Predicate is deterministic */
 #define P_AUTOLOAD		(0x08000000) /* autoload/2 explicit import */
 #define P_WAITED_FOR		(0x10000000) /* Someone is waiting for this predicate */
 #define	P_LOCKED_SUPERVISOR	(0x20000000) /* Fixed supervisor */
@@ -900,6 +900,7 @@ with one operation, it turns out to be faster as well.
 					 /* from predicate */
 #define SSU_COMMIT_CLAUSE	(0x0100) /* Head => Body */
 #define SSU_CHOICE_CLAUSE	(0x0200) /* Head ?=> Body */
+#define CL_HEAD_TERMS		(0x0400) /* Head contains terms used in body */
 
 #define CLAUSE_TYPE_MASK (UNIT_CLAUSE|SSU_COMMIT_CLAUSE|SSU_CHOICE_CLAUSE)
 
@@ -966,11 +967,14 @@ Macros for environment frames (local stack frames)
 #define FR_CLEANUP		(0x0100) /* setup_call_cleanup/4 */
 #define FR_INRESET		(0x0200) /* Continuations: inside reset/3 */
 #define FR_SSU_DET		(0x0400) /* Demands det on => rules */
+#define FR_DET			(0x0800) /* Declared det */
+#define FR_DETGUARD		(0x1000) /* Frame is guarded for determinism */
+#define FR_DETGUARD_SET		(0x2000) /* Flag was set on this frame */
 #define FR_WATCHED (FR_CLEANUP|FR_DEBUG)
 
-#define FR_MAGIC_MASK		(0xfffff000)
-#define FR_MAGIC_MASK2		(0xffff0000)
-#define FR_MAGIC		(0x549d5000)
+#define FR_MAGIC_MASK		(0xffff0000)
+#define FR_MAGIC_MASK2		(0xfff00000)
+#define FR_MAGIC		(0xc9d50000)
 
 #define isFrame(fr)		(((fr)->flags&FR_MAGIC_MASK) == FR_MAGIC)
 #define wasFrame(fr)		(((fr)->flags&FR_MAGIC_MASK2) == \
@@ -1045,9 +1049,11 @@ typedef uint64_t lgen_t;
 
 #define setGenerationFrame(fr) setGenerationFrame__LD((fr) PASS_LD)
 
-#define FR_CLEAR_NEXT	(FR_SKIPPED|FR_WATCHED|FR_CATCHED|\
+#define FR_LCO_CLEAR	(FR_SKIPPED|FR_WATCHED|FR_CATCHED|\
 			 FR_HIDE_CHILDS|FR_CLEANUP|FR_SSU_DET)
-#define FR_CLEAR_FLAGS	(FR_CLEAR_NEXT|FR_CONTEXT)
+#define FR_CLEAR_NEXT	(FR_LCO_CLEAR|FR_DET|FR_DETGUARD)
+#define FR_CLEAR_ALWAYS (FR_CONTEXT|FR_DETGUARD_SET)
+#define FR_CLEAR_FLAGS	(FR_CLEAR_NEXT|FR_CLEAR_ALWAYS)
 
 #define setNextFrameFlags(next, fr) \
 	do \
@@ -1055,6 +1061,15 @@ typedef uint64_t lgen_t;
 	  (next)->flags = ((fr)->flags) & ~FR_CLEAR_FLAGS; \
 	} while(0)
 
+#define lcoSetNextFrameFlags2(next, fr) \
+	do \
+	{ (next)->level = (fr)->level+1; \
+	  (next)->flags = ((fr)->flags) & ~(FR_LCO_CLEAR|FR_CLEAR_ALWAYS); \
+	} while(0)
+
+#define lcoSetNextFrameFlags(fr) \
+	lcoSetNextFrameFlags2(fr,fr)
+
 #define setFramePredicate(fr, def) \
 	do \
 	{ (fr)->predicate = (def); \
