@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2016-2020, VU University Amsterdam
+    Copyright (c)  2016-2021, VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -201,6 +202,7 @@ typedef struct idg_node
   unsigned	aborted : 1;		/* re-evaluation was aborted */
   unsigned	monotonic : 1;		/* Associated predicate is monotonic */
   unsigned	lazy : 1;		/* Lazy monotonic node */
+  unsigned	lazy_queued : 1;	/* There are answers queued */
   unsigned	force_reeval : 1;	/* Forced reevaluation for monotonic */
   int		falsecount;		/* Invalidate count */
 #ifdef O_TRIE_STATS
@@ -276,6 +278,7 @@ COMMON(void)	merge_tabling_trail(tbl_trail *into, tbl_trail *from);
 #define TP_SHARED	(0x0004)	/* Shared tabling */
 #define TP_OPAQUE	(0x0008)	/* Declared opaque */
 #define TP_LAZY		(0x0010)	/* Lazy (monotonic) */
+#define TP_INCREMENTAL	(0x0020)	/* Incremental tabling */
 
 typedef struct table_props
 { unsigned int	flags;			/* TP_* flags */
@@ -310,4 +313,5 @@ COMMON(int)	tbl_is_restraint_flag(atom_t key);
 COMMON(int)	tbl_get_restraint_flag(term_t t, atom_t key ARG_LD);
 COMMON(int)	tbl_set_restraint_flag(term_t t, atom_t key ARG_LD);
 COMMON(int)	setMonotonicMode(atom_t a);
+COMMON(void)	tbl_set_incremental_predicate(Definition def, int val);
 #endif /*_PL_TABLING_H*/
